ConstEvalCtxt

Struct ConstEvalCtxt 

pub struct ConstEvalCtxt<'tcx> { /* private fields */ }
The context required to evaluate a constant expression.

This is currently limited to constant folding and reading the value of named constants.

See the module level documentation for some context.

Implementations

Source

impl<'tcx> ConstEvalCtxt<'tcx>

pub fn new(cx: &LateContext<'tcx>) -> Self

Creates the evaluation context from the lint context. This requires the lint context to be in a body (i.e. cx.enclosing_body.is_some()).

pub fn with_env( tcx: TyCtxt<'tcx>, typing_env: TypingEnv<'tcx>, typeck: &'tcx TypeckResults<'tcx>, ) -> Self

Creates an evaluation context.

pub fn eval_with_source( &self, e: &Expr<'_>, ctxt: SyntaxContext, ) -> Option<(Constant, ConstantSource)>

Attempts to evaluate the expression and returns both the value and whether it’s dependant on other items.

pub fn eval(&self, e: &Expr<'_>) -> Option<Constant>

Attempts to evaluate the expression.

pub fn eval_local(&self, e: &Expr<'_>, ctxt: SyntaxContext) -> Option<Constant>

Attempts to evaluate the expression without accessing other items.

The context argument is the context used to view the evaluated expression. e.g. when evaluating the argument in f(m!(1)) the context of the call expression should be used. This is need so the const evaluator can see the m macro and marke the evaluation as non-local independant of what the macro expands to.

pub fn eval_full_int( &self, e: &Expr<'_>, ctxt: SyntaxContext, ) -> Option<FullInt>

Attempts to evaluate the expression as an integer without accessing other items.

The context argument is the context used to view the evaluated expression. e.g. when evaluating the argument in f(m!(1)) the context of the call expression should be used. This is need so the const evaluator can see the m macro and marke the evaluation as non-local independant of what the macro expands to.

pub fn eval_pat_expr(&self, pat_expr: &PatExpr<'_>) -> Option<Constant>

pub fn eval_is_empty(&self, e: &Expr<'_>) -> Option<bool>

Simple constant folding to determine if an expression is an empty slice, str, array, … None will be returned if the constness cannot be determined, or if the resolution leaves the local crate.